----------------------------------------------------------------------------
-- Script: 	Auto Roll                                                      --
-- Version:	1.4                                                          --
-- Last update: 5.12.2002 
-- Modified on : 21.04.2003                                               --
-- By:   Antoine Pasquay                                           --
-- Web:	www.athanaze.com                                                --
-- E-mail: athanaze@fr.st     --
----------------------------------------------------------------------------
-- AUTO ROLL
----------------------------------------------------------------------------
-- Description :
-- Permet de faire tourner une roue proportionnellement a son dplacement absolu.
--
-- Historique :
-- V1.0 : fonction "Rduire cls".
-- V1.0 : Slection automatique de l'intervalle de temps
--      : Annulable via Ctrl + Z
-- v1.1 : Bouton annuler sur panneau
-- V1.2 : Contrleur type Euler  et piste patinage.
-- V1.3 : Dtection automatique du sens de rotation 
-- V1.4 : Seuil de dtection de changement de sens
-- Par Adam Wierzchowski" 
-- E-mail: asd@grafik.3D.pl 
-- Web: www.asd.3D.pl

--------------------------------------
-- $$$$ ROLLIN $$$$
--------------------------------------
rollout autoroll "Auto Roll"
(
	group "Objet"
	(
	pickbutton selobj "Choisir Roue" width:140
	label lab_rotaxis "Axe de rotation: " across:1
	radiobuttons rot_axis  labels:#("X","Y","Z") default:3 columns: 3
	spinner obj_radius "Rayon: " type:#float range:[1,1000,48] fieldwidth:45 align:#left
	checkbox detect "Dtecter rayon" width:140 enabled:false
	button center "Pivot au centre" width:140 enabled:false
	)
	group "Rotation"
	(
	checkbox autorot "Inversion auto." align:#left checked: true
	spinner rev_angle "Angle reverse: " type:#float range:[0,180,90] fieldwidth:45 align:#left
	checkbox reverse "Forc inverse" align:#left
	checkbox seteuler "Assigner Euler" align:#left 
	)
	group "Calcul"
	(
	spinner calc_start "Dbut intervalle: " type:#integer range:[-9999,9998,animationRange.start] fieldwidth:47 align:#right
	spinner calc_end "Fin intervalle: " type:#integer range:[-9998,9999,animationRange.end] fieldwidth:47 align:#right
	label t1 "Calculer toutes les"
	spinner step "N image: " type:#integer range:[1,50,1] fieldwidth:47 align:#right
	checkbox reduc_key "Rduire cl" checked: false
	spinner treshold "Seuil:"   width:60 range:[0,1000,1]   align:#right
	button go "ROULER !" enabled:false width:140
	button cancl "Annuler" width:140
	)
		
on autoroll open do
(
global xvector, yvector,zvector
global xradius, yradius,zradius
xvector = [0,1,1]
yvector = [1,0,1]
zvector = [1,1,0]
global sens = -1
)

on selobj picked obj do
(
if (superClassOf obj == geometryClass) then
	(
	global animated_object
	animated_object = obj
	selobj.text = obj.name
	detect.enabled = true
	detect.state = false
	center.enabled = true
	go.enabled = true
	)
)

on go pressed do
(
sens = -1
undo "Rollin" on
(
animate on
	(
	if seteuler.checked then
		(
		c = euler_xyz()
		animated_object.rotation.controller = c
		)
	cont = animated_object.rotation.controller
	for frame in animationRange.start to animationRange.end do
		(
		key_index = getkeyindex cont frame
		if (key_index) != 0 and (key_index) != 1 then (deleteKey cont key_index)
		)
	for frame in calc_start.value to calc_end.value by step.value do
		(
		if frame < step.value then (
			at time 0 (prev_pos = animated_object.pos)
			)
		else (
			at time (frame-step.value) (prev_pos = animated_object.pos))
			at time frame (current_pos = animated_object.pos)
			at time (frame-2*step.value) (next_pos = animated_object.pos)
		covered_distance = distance prev_pos current_pos
		Dp = dot (normalize (current_pos - prev_pos)) (normalize (prev_pos - next_pos))
		if (Dp < cos rev_angle.value)and (autorot.checked) then 
			( 
			sens = sens * -1
			)
		rotation = (sens * 180 * covered_distance)/(pi*obj_radius.value)
		if reverse.checked then (rotation = - rotation) 
		in coordsys local at time frame
			(
			case rot_axis.state of
				(
				1: rotate animated_object (eulerangles rotation 0 0)
				2: rotate animated_object (eulerangles 0 rotation 0)
				3: rotate animated_object (eulerangles 0 0 rotation)
				)
			)
		)		
	)
	if reduc_key.checked == true then reduceKeys animated_object.rotation.controller treshold.value 1f calc_start.value calc_end.value
)
)

on center pressed do
(
addModifier animated_object (edit_mesh())
obj_center = [0,0,0]
for i in 1 to animated_object.numVerts do
	(
	obj_center += getVert animated_object i
	)
obj_center /= animated_object.numVerts
animated_object.pivot = obj_center
deleteModifier animated_object 1
)

on detect changed new_state do
(
if new_state == true then
	(
	addModifier animated_object (edit_mesh())
	obj_center = [0,0,0]
	for i in 1 to animated_object.numVerts do
		(
		obj_center += in coordsys local getVert animated_object i
		)
	obj_center /= animated_object.numVerts
	f_xvert_index = 1
	f_yvert_index = 1
	f_zvert_index = 1
	for i in 2 to animated_object.numVerts do
		(
		f_xvert_pos = in coordsys local getVert animated_object f_xvert_index
		f_yvert_pos = in coordsys local getVert animated_object f_yvert_index
		f_zvert_pos = in coordsys local getVert animated_object f_zvert_index
		new_vert_pos = in coordsys local getVert animated_object i
		if (distance (obj_center*xvector) (new_vert_pos*xvector)) > (distance (obj_center*xvector) (f_xvert_pos*xvector)) then f_xvert_index = i
		if (distance (obj_center*yvector) (new_vert_pos*yvector)) > (distance (obj_center*yvector) (f_yvert_pos*yvector)) then f_yvert_index = i
		if (distance (obj_center*zvector) (new_vert_pos*zvector)) > (distance (obj_center*zvector) (f_zvert_pos*zvector)) then f_zvert_index = i
		)
	f_xvert_pos = in coordsys local getVert animated_object f_xvert_index
	f_yvert_pos = in coordsys local getVert animated_object f_yvert_index
	f_zvert_pos = in coordsys local getVert animated_object f_zvert_index
	xradius = (distance (obj_center*xvector) (f_xvert_pos*xvector))
	yradius = (distance (obj_center*yvector) (f_yvert_pos*yvector))
	zradius = (distance (obj_center*zvector) (f_zvert_pos*zvector))
	deleteModifier animated_object 1
	case rot_axis.state of
		(
		1: obj_radius.value = xradius
		2: obj_radius.value = yradius
		3: obj_radius.value = zradius
		)
	)
)

on calc_start changed val do
(
if val > calc_end.value then calc_end.value = val
)

on calc_end changed val do
(
if val < calc_start.value then calc_start.value = val
)

on rot_axis changed state do
(
if (state == 1) and (detect.state == true) then obj_radius.value = xradius
if (state == 2) and (detect.state == true) then obj_radius.value = yradius
if (state == 3) and (detect.state == true) then obj_radius.value = zradius
)

on seteuler changed state do
	(
	if seteuler.checked then pattrack.enabled = true else pattrack.enabled = false
	)
on cancl pressed do
	( 
	max undo
	if reduc_key.checked == true then max undo
	)

)
--------------------------------------------
--  $$$$ INVERSEUR $$$$
--------------------------------------------
rollout roabout "A propos"
(
group ""
	(
	label lab1 "Auto Roll" align:#center
	label lab2 "Versoon 1.0" align:#center
	label copyright_a "par antoine Pasquay"
	hyperlink label_l09 "www.athanaze.com" align:#center address:"http://www.athanaze.fr.st/index.php?scripts" color:(color 70 70 255) hoverColor:(color 230 63 63)
	label l10 "E-mail :"
	hyperlink label_l11 "athanaze@fr.st" align:#center address:"mailto:athanaze@fr.st" color:(color 70 70 255) hoverColor:(color 230 63 63)

	)
)
--------------------------------------

Rollfloater = newRolloutFloater "Auto Roll" 200 515
addrollout autoroll Rollfloater
addrollout roabout Rollfloater rolledup: true